@@ -1574,12 +1574,35 @@ style_context_changed (GtkStyleContext     *style_context,
 {
 	PangoFontDescription *font_desc = NULL;
 
-	gtk_style_context_get (style_context, GTK_STATE_FLAG_NORMAL,
+	gtk_style_context_save (style_context);
+	gtk_style_context_set_state (style_context, GTK_STATE_FLAG_NORMAL);
+
+	gtk_style_context_get (style_context,
+			       gtk_style_context_get_state (style_context),
 			       GTK_STYLE_PROPERTY_FONT, &font_desc,
 			       NULL);
+
+	gtk_style_context_restore (style_context);
+
+	/*
+	 * Work around issue where when a proposal provides "<b>markup</b>" and
+	 * the weight is set in the font description, the <b> markup will not
+	 * have it's weight respected. This seems to be happening because the
+	 * weight mask is getting set in pango_font_description_from_string()
+	 * even if the the value is set to normal. That matter is complicated
+	 * because PangoAttrFontDesc and PangoAttrWeight will both have the
+	 * same starting offset in the PangoLayout.
+	 * https://bugzilla.gnome.org/show_bug.cgi?id=755968
+	 */
+	if (PANGO_WEIGHT_NORMAL == pango_font_description_get_weight (font_desc))
+	{
+		pango_font_description_unset_fields (font_desc, PANGO_FONT_MASK_WEIGHT);
+	}
+
 	g_object_set (completion->priv->cell_renderer_proposal,
 		      "font-desc", font_desc,
 		      NULL);
+
 	pango_font_description_free (font_desc);
 }
 
@@ -1959,6 +1982,7 @@ cell_icon_func (GtkTreeViewColumn *column,
 	GdkPixbuf *pixbuf;
 	gchar *icon_name;
 	GIcon *gicon;
+	gboolean set = FALSE;
 
 	gtk_tree_model_get (model, iter,
 	                    GTK_SOURCE_COMPLETION_MODEL_COLUMN_ICON, &pixbuf,
@@ -1970,18 +1994,26 @@ cell_icon_func (GtkTreeViewColumn *column,
 	{
 		g_object_set (cell, "pixbuf", pixbuf, NULL);
 		g_object_unref (pixbuf);
+		set = TRUE;
 	}
 
 	if (icon_name != NULL)
 	{
 		g_object_set (cell, "icon-name", icon_name, NULL);
 		g_free (icon_name);
+		set = TRUE;
 	}
 
 	if (gicon != NULL)
 	{
 		g_object_set (cell, "gicon", gicon, NULL);
 		g_object_unref (gicon);
+		set = TRUE;
+	}
+
+	if (!set)
+	{
+		g_object_set (cell, "icon-name", NULL, NULL);
 	}
 }
 
@@ -2044,11 +2076,20 @@ init_tree_view (GtkSourceCompletion *completion,
 
 	style_context = gtk_widget_get_style_context (GTK_WIDGET (completion->priv->tree_view_proposals));
 
+	gtk_style_context_save (style_context);
+	gtk_style_context_set_state (style_context, GTK_STATE_FLAG_INSENSITIVE);
+
 	gtk_style_context_get (style_context,
-			       GTK_STATE_FLAG_INSENSITIVE,
+			       gtk_style_context_get_state (style_context),
 			       "background-color", &background_color,
 			       NULL);
 
+	gtk_style_context_get_color (style_context,
+				     gtk_style_context_get_state (style_context),
+				     &foreground_color);
+
+	gtk_style_context_restore (style_context);
+
 	g_object_set (cell_renderer,
 	              "cell-background-rgba", &background_color,
 	              NULL);
@@ -2070,10 +2111,6 @@ init_tree_view (GtkSourceCompletion *completion,
 					     "foreground-set", GTK_SOURCE_COMPLETION_MODEL_COLUMN_IS_HEADER,
 					     NULL);
 
-	gtk_style_context_get_color (style_context,
-	                             GTK_STATE_FLAG_INSENSITIVE,
-	                             &foreground_color);
-
 	g_object_set (cell_renderer,
 	              "foreground-rgba", &foreground_color,
 	              "cell-background-rgba", &background_color,
