@@ -89,6 +89,10 @@ struct _GtkSourceGutterPrivate
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceGutter, gtk_source_gutter, G_TYPE_OBJECT)
 
+static gboolean on_view_draw (GtkSourceView   *view,
+                              cairo_t         *cr,
+                              GtkSourceGutter *gutter);
+
 static gboolean on_view_motion_notify_event (GtkSourceView   *view,
                                              GdkEventMotion  *event,
                                              GtkSourceGutter *gutter);
@@ -285,6 +289,12 @@ set_view (GtkSourceGutter *gutter,
 {
 	gutter->priv->view = view;
 
+	g_signal_connect_object (view,
+				 "draw",
+				 G_CALLBACK (on_view_draw),
+				 gutter,
+				 G_CONNECT_AFTER);
+
 	g_signal_connect_object (view,
 				 "motion-notify-event",
 				 G_CALLBACK (on_view_motion_notify_event),
@@ -1217,10 +1227,10 @@ end_draw (GtkSourceGutter *gutter)
 	}
 }
 
-void
-gtk_source_gutter_draw (GtkSourceGutter *gutter,
-                        GtkSourceView   *view,
-                        cairo_t         *cr)
+static gboolean
+on_view_draw (GtkSourceView   *view,
+              cairo_t         *cr,
+              GtkSourceGutter *gutter)
 {
 	GdkRectangle clip;
 	GtkTextView *text_view;
@@ -1234,7 +1244,7 @@ gtk_source_gutter_draw (GtkSourceGutter *gutter,
 
 	if (!get_clip_rectangle (gutter, view, cr, &clip))
 	{
-		return;
+		return GDK_EVENT_PROPAGATE;
 	}
 
 	gutter->priv->is_drawing = TRUE;
@@ -1291,6 +1301,8 @@ gtk_source_gutter_draw (GtkSourceGutter *gutter,
 
 	g_array_free (renderer_widths, TRUE);
 	lines_info_free (info);
+
+	return GDK_EVENT_PROPAGATE;
 }
 
 static Renderer *
