@@ -37,8 +37,6 @@ struct _GtkSourceGutterRendererTextPrivate
 	gchar *text;
 
 	PangoLayout *cached_layout;
-	PangoAttribute *fg_attr;
-	PangoAttrList *cached_attr_list;
 
 	guint is_markup : 1;
 };
@@ -53,46 +51,20 @@ enum
 };
 
 static void
-create_layout (GtkSourceGutterRendererText *renderer,
-               GtkWidget                   *widget)
-{
-	PangoLayout *layout;
-	PangoAttribute *attr;
-	GtkStyleContext *context;
-	GdkRGBA color;
-	PangoAttrList *attr_list;
-
-	layout = gtk_widget_create_pango_layout (widget, NULL);
-
-	context = gtk_widget_get_style_context (widget);
-	gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &color);
-
-	attr = pango_attr_foreground_new (color.red * 65535,
-	                                  color.green * 65535,
-	                                  color.blue * 65535);
-
-	attr->start_index = 0;
-	attr->end_index = G_MAXINT;
-
-	attr_list = pango_attr_list_new ();
-	pango_attr_list_insert (attr_list, attr);
-
-	renderer->priv->fg_attr = attr;
-	renderer->priv->cached_layout = layout;
-	renderer->priv->cached_attr_list = attr_list;
-}
-
-static void
-gutter_renderer_text_begin (GtkSourceGutterRenderer      *renderer,
-                            cairo_t                      *cr,
-                            GdkRectangle                 *background_area,
-                            GdkRectangle                 *cell_area,
-                            GtkTextIter                  *start,
-                            GtkTextIter                  *end)
+gutter_renderer_text_begin (GtkSourceGutterRenderer *renderer,
+			    cairo_t                 *cr,
+			    GdkRectangle            *background_area,
+			    GdkRectangle            *cell_area,
+			    GtkTextIter             *start,
+			    GtkTextIter             *end)
 {
 	GtkSourceGutterRendererText *text = GTK_SOURCE_GUTTER_RENDERER_TEXT (renderer);
+	GtkTextView *view;
 
-	create_layout (text, GTK_WIDGET (gtk_source_gutter_renderer_get_view (renderer)));
+	view = gtk_source_gutter_renderer_get_view (renderer);
+
+	g_clear_object (&text->priv->cached_layout);
+	text->priv->cached_layout = gtk_widget_create_pango_layout (GTK_WIDGET (view), NULL);
 
 	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->begin != NULL)
 	{
@@ -106,20 +78,17 @@ gutter_renderer_text_begin (GtkSourceGutterRenderer      *renderer,
 }
 
 static void
-center_on (GtkSourceGutterRenderer *renderer,
-           GdkRectangle            *cell_area,
-           GtkTextIter             *iter,
-           gint                     width,
-           gint                     height,
-           gfloat                   xalign,
-           gfloat                   yalign,
-           gint                    *x,
-           gint                    *y)
+center_on (GtkTextView  *view,
+           GdkRectangle *cell_area,
+           GtkTextIter  *iter,
+           gint          width,
+           gint          height,
+           gfloat        xalign,
+           gfloat        yalign,
+           gint         *x,
+           gint         *y)
 {
 	GdkRectangle location;
-	GtkTextView *view;
-
-	view = gtk_source_gutter_renderer_get_view (renderer);
 
 	gtk_text_view_get_iter_location (view, iter, &location);
 
@@ -129,21 +98,20 @@ center_on (GtkSourceGutterRenderer *renderer,
 
 static void
 gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
-                           cairo_t                      *cr,
-                           GdkRectangle                 *background_area,
-                           GdkRectangle                 *cell_area,
-                           GtkTextIter                  *start,
-                           GtkTextIter                  *end,
-                           GtkSourceGutterRendererState  state)
+			   cairo_t                      *cr,
+			   GdkRectangle                 *background_area,
+			   GdkRectangle                 *cell_area,
+			   GtkTextIter                  *start,
+			   GtkTextIter                  *end,
+			   GtkSourceGutterRendererState  state)
 {
 	GtkSourceGutterRendererText *text = GTK_SOURCE_GUTTER_RENDERER_TEXT (renderer);
+	GtkTextView *view;
 	gint width;
 	gint height;
-	PangoAttrList *attr_list;
 	gfloat xalign;
 	gfloat yalign;
 	GtkSourceGutterRendererAlignmentMode mode;
-	GtkTextView *view;
 	gint x = 0;
 	gint y = 0;
 	GtkStyleContext *context;
@@ -175,26 +143,21 @@ gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
 		                       -1);
 	}
 
-	attr_list = pango_layout_get_attributes (text->priv->cached_layout);
-
-	if (!attr_list)
-	{
-		pango_layout_set_attributes (text->priv->cached_layout,
-		                             pango_attr_list_copy (text->priv->cached_attr_list));
-	}
-	else
-	{
-		pango_attr_list_insert (attr_list,
-		                        pango_attribute_copy (text->priv->fg_attr));
-	}
-
 	pango_layout_get_pixel_size (text->priv->cached_layout, &width, &height);
 
 	gtk_source_gutter_renderer_get_alignment (renderer,
 	                                          &xalign,
 	                                          &yalign);
 
-	mode = gtk_source_gutter_renderer_get_alignment_mode (renderer);
+	/* Avoid calculations if we don't wrap text */
+	if (gtk_text_view_get_wrap_mode (view) == GTK_WRAP_NONE)
+	{
+		mode = GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL;
+	}
+	else
+	{
+		mode = gtk_source_gutter_renderer_get_alignment_mode (renderer);
+	}
 
 	switch (mode)
 	{
@@ -202,8 +165,9 @@ gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
 			x = cell_area->x + (cell_area->width - width) * xalign;
 			y = cell_area->y + (cell_area->height - height) * yalign;
 			break;
+
 		case GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_FIRST:
-			center_on (renderer,
+			center_on (view,
 			           cell_area,
 			           start,
 			           width,
@@ -213,8 +177,9 @@ gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
 			           &x,
 			           &y);
 			break;
+
 		case GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_LAST:
-			center_on (renderer,
+			center_on (view,
 			           cell_area,
 			           end,
 			           width,
@@ -224,6 +189,7 @@ gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
 			           &x,
 			           &y);
 			break;
+
 		default:
 			g_assert_not_reached ();
 	}
@@ -239,11 +205,6 @@ gutter_renderer_text_end (GtkSourceGutterRenderer *renderer)
 
 	g_clear_object (&text->priv->cached_layout);
 
-	pango_attr_list_unref (text->priv->cached_attr_list);
-	text->priv->cached_attr_list = NULL;
-
-	text->priv->fg_attr = NULL;
-
 	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->end != NULL)
 	{
 		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->end (renderer);
@@ -257,26 +218,20 @@ measure_text (GtkSourceGutterRendererText *renderer,
               gint                        *width,
               gint                        *height)
 {
-	PangoLayout *layout;
-	GtkSourceGutterRenderer *r;
 	GtkTextView *view;
+	PangoLayout *layout;
 
-	r = GTK_SOURCE_GUTTER_RENDERER (renderer);
-	view = gtk_source_gutter_renderer_get_view (r);
+	view = gtk_source_gutter_renderer_get_view (GTK_SOURCE_GUTTER_RENDERER (renderer));
 
 	layout = gtk_widget_create_pango_layout (GTK_WIDGET (view), NULL);
 
-	if (markup)
+	if (markup != NULL)
 	{
-		pango_layout_set_markup (layout,
-		                         markup,
-		                         -1);
+		pango_layout_set_markup (layout, markup, -1);
 	}
 	else
 	{
-		pango_layout_set_text (layout,
-		                       text,
-		                       -1);
+		pango_layout_set_text (layout, text, -1);
 	}
 
 	pango_layout_get_pixel_size (layout, width, height);
@@ -286,10 +241,12 @@ measure_text (GtkSourceGutterRendererText *renderer,
 
 /**
  * gtk_source_gutter_renderer_text_measure:
- * @renderer: A #GtkSourceGutterRendererText
- * @text: the text to measure
- * @width: (out): The width of the text in pixels
- * @height: (out): The height of the text in pixels
+ * @renderer: a #GtkSourceGutterRendererText.
+ * @text: the text to measure.
+ * @width: (out) (optional): location to store the width of the text in pixels,
+ *   or %NULL.
+ * @height: (out) (optional): location to store the height of the text in
+ *   pixels, or %NULL.
  *
  * Measures the text provided using the pango layout used by the
  * #GtkSourceGutterRendererText.
@@ -308,10 +265,12 @@ gtk_source_gutter_renderer_text_measure (GtkSourceGutterRendererText *renderer,
 
 /**
  * gtk_source_gutter_renderer_text_measure_markup:
- * @renderer: A #GtkSourceGutterRendererText.
- * @markup: the pango markup to measure
- * @width: (out): The width of the text in pixels
- * @height: (out): The height of the text in pixels
+ * @renderer: a #GtkSourceGutterRendererText.
+ * @markup: the pango markup to measure.
+ * @width: (out) (optional): location to store the width of the text in pixels,
+ *   or %NULL.
+ * @height: (out) (optional): location to store the height of the text in
+ *   pixels, or %NULL.
  *
  * Measures the pango markup provided using the pango layout used by the
  * #GtkSourceGutterRendererText.
@@ -334,6 +293,7 @@ gtk_source_gutter_renderer_text_finalize (GObject *object)
 	GtkSourceGutterRendererText *renderer = GTK_SOURCE_GUTTER_RENDERER_TEXT (object);
 
 	g_free (renderer->priv->text);
+	g_clear_object (&renderer->priv->cached_layout);
 
 	G_OBJECT_CLASS (gtk_source_gutter_renderer_text_parent_class)->finalize (object);
 }
