@@ -4,30 +4,29 @@
  * Copyright (C) 2015 Christian Hergert <christian@hergert.me>
  * Copyright (C) 2015 Ignacio Casal Quinteiro <icq@gnome.org>
  *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * GtkSourceView is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * GtkSourceView is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gtksourcemap.h"
+#include <string.h>
 #include "gtksourcebuffer.h"
 #include "gtksourcecompletion.h"
-#include "gtksourcemap.h"
 #include "gtksourcestyle-private.h"
 #include "gtksourcestylescheme.h"
 #include "gtksourceview-utils.h"
 
-#include <glib/gi18n.h>
-#include <string.h>
-
 /**
  * SECTION:map
  * @Short_description: Widget that displays a map for a specific #GtkSourceView
@@ -348,8 +347,11 @@ gtk_source_map_rebuild_css (GtkSourceMap *map)
 		context = gtk_widget_get_style_context (GTK_WIDGET (priv->view));
 		gtk_style_context_save (context);
 		gtk_style_context_add_class (context, "view");
+		gtk_style_context_set_state (context, GTK_STATE_FLAG_SELECTED);
 		G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-		gtk_style_context_get_background_color (context, GTK_STATE_FLAG_SELECTED, &color);
+		gtk_style_context_get_background_color (context,
+							gtk_style_context_get_state (context),
+							&color);
 		G_GNUC_END_IGNORE_DEPRECATIONS;
 		gtk_style_context_restore (context);
 		background = gdk_rgba_to_string (&color);
@@ -730,6 +732,13 @@ connect_view (GtkSourceMap  *map,
 		gtk_widget_add_events (GTK_WIDGET (priv->view), GDK_LEAVE_NOTIFY_MASK);
 	}
 
+	/* If we are not visible, we want to block certain signal handlers */
+	if (!gtk_widget_get_visible (GTK_WIDGET (map)))
+	{
+		g_signal_handler_block (vadj, priv->view_vadj_value_changed_handler);
+		g_signal_handler_block (vadj, priv->view_vadj_notify_upper_handler);
+	}
+
 	gtk_source_map_rebuild_css (map);
 }
 
@@ -1052,6 +1061,48 @@ gtk_source_map_realize (GtkWidget *widget)
 	set_view_cursor (GTK_SOURCE_MAP (widget));
 }
 
+static void
+gtk_source_map_show (GtkWidget *widget)
+{
+	GtkSourceMap *map = GTK_SOURCE_MAP (widget);
+	GtkSourceMapPrivate *priv;
+	GtkAdjustment *vadj;
+
+	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->show (widget);
+
+	priv = gtk_source_map_get_instance_private (map);
+
+	if (priv->view != NULL)
+	{
+		vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (priv->view));
+
+		g_signal_handler_unblock (vadj, priv->view_vadj_value_changed_handler);
+		g_signal_handler_unblock (vadj, priv->view_vadj_notify_upper_handler);
+
+		g_object_notify (G_OBJECT (vadj), "upper");
+		g_signal_emit_by_name (vadj, "value-changed");
+	}
+}
+
+static void
+gtk_source_map_hide (GtkWidget *widget)
+{
+	GtkSourceMap *map = GTK_SOURCE_MAP (widget);
+	GtkSourceMapPrivate *priv;
+	GtkAdjustment *vadj;
+
+	GTK_WIDGET_CLASS (gtk_source_map_parent_class)->hide (widget);
+
+	priv = gtk_source_map_get_instance_private (map);
+
+	if (priv->view != NULL)
+	{
+		vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (priv->view));
+		g_signal_handler_block (vadj, priv->view_vadj_value_changed_handler);
+		g_signal_handler_block (vadj, priv->view_vadj_notify_upper_handler);
+	}
+}
+
 static void
 gtk_source_map_class_init (GtkSourceMapClass *klass)
 {
@@ -1065,25 +1116,27 @@ gtk_source_map_class_init (GtkSourceMapClass *klass)
 	widget_class->draw = gtk_source_map_draw;
 	widget_class->get_preferred_height = gtk_source_map_get_preferred_height;
 	widget_class->get_preferred_width = gtk_source_map_get_preferred_width;
+	widget_class->hide = gtk_source_map_hide;
 	widget_class->size_allocate = gtk_source_map_size_allocate;
 	widget_class->button_press_event = gtk_source_map_button_press_event;
 	widget_class->button_release_event = gtk_source_map_button_release_event;
 	widget_class->motion_notify_event = gtk_source_map_motion_notify_event;
 	widget_class->scroll_event = gtk_source_map_scroll_event;
+	widget_class->show = gtk_source_map_show;
 	widget_class->state_flags_changed = gtk_source_map_state_flags_changed;
 	widget_class->realize = gtk_source_map_realize;
 
 	pspecs[PROP_VIEW] =
 		g_param_spec_object ("view",
-		                     _("View"),
-		                     _("The view this widget is mapping."),
+		                     "View",
+		                     "The view this widget is mapping.",
 		                     GTK_SOURCE_TYPE_VIEW,
 		                     (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	pspecs[PROP_FONT_DESC] =
 		g_param_spec_boxed ("font-desc",
-		                    _("Font Description"),
-		                    _("The Pango font description to use."),
+		                    "Font Description",
+		                    "The Pango font description to use.",
 		                    PANGO_TYPE_FONT_DESCRIPTION,
 		                    (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
